package fr.inria.phoenix.diasuite.framework.misc;

import fr.inria.diagen.core.deploy.AbstractDeploy;

import fr.inria.phoenix.diasuite.framework.context.averagetemperature.AbstractAverageTemperature;
import fr.inria.phoenix.diasuite.framework.context.hvacregulation.AbstractHVACRegulation;

import fr.inria.phoenix.diasuite.framework.controller.hvaccontroller.AbstractHVACController;

/**
 * This class should be implemented to bind the implementation of the various components
 */
public abstract class AppComponentBinder extends AbstractDeploy {

    // Context instances
    private AbstractAverageTemperature averageTemperatureInstance = null;
    private AbstractHVACRegulation hVACRegulationInstance = null;

    // Controller instances
    private AbstractHVACController hVACControllerInstance = null;
    
    @Override
    public void deployAll() {
        // Initialization of contexts
        if (averageTemperatureInstance == null)
            averageTemperatureInstance = getInstance(getAverageTemperatureClass());
        if (hVACRegulationInstance == null)
            hVACRegulationInstance = getInstance(getHVACRegulationClass());
        // Intialization of controllers
        if (hVACControllerInstance == null)
            hVACControllerInstance = getInstance(getHVACControllerClass());
        // Deploying contexts
        deploy(averageTemperatureInstance);
        deploy(hVACRegulationInstance);
        // Deploying controllers
        deploy(hVACControllerInstance);
    }
    
    @Override
    public void undeployAll() {
        // Undeploying contexts
        undeploy(averageTemperatureInstance);
        undeploy(hVACRegulationInstance);
        // Undeploying controllers
        undeploy(hVACControllerInstance);
    }
    
    // Abstract binding methods for contexts
    /**
     * Overrides this method to provide the implementation class of the <code>AverageTemperature</code> context
     * 
     * <pre>
     * context AverageTemperature as Float indexed by location as String {
     * 	when provided temperature from TemperatureSensor
     * 	get temperature from TemperatureSensor
     * 	always publish;
     * }
     * </pre>
     * @return a class object of a derivation of {@link AbstractAverageTemperature} that implements the <code>AverageTemperature</code> context
     */
    public abstract Class<? extends AbstractAverageTemperature> getAverageTemperatureClass();
    
    /**
     * Overrides this method to provide the implementation class of the <code>HVACRegulation</code> context
     * 
     * <pre>
     * context HVACRegulation as TemperatureLevel indexed by location as String {
     * 	when provided AverageTemperature 
     * 	get targetTemperature from ControlPanel,
     * 		status from HVACSystem
     * 	maybe publish;
     * }
     * </pre>
     * @return a class object of a derivation of {@link AbstractHVACRegulation} that implements the <code>HVACRegulation</code> context
     */
    public abstract Class<? extends AbstractHVACRegulation> getHVACRegulationClass();
    
    // End of abstract binding methods for contexts
    
    // Abstract binding methods for controllers
    /**
     * Overrides this method to provide the implementation class of the <code>HVACController</code> controller
     * 
     * <pre>
     * controller HVACController {
     * 	when provided HVACRegulation do Regulate on HVACSystem;
     * }
     * </pre>
     * @return a class object of a derivation of {@link AbstractHVACController} that implements the <code>HVACController</code> controller
     */
    public abstract Class<? extends AbstractHVACController> getHVACControllerClass();
    
    // End of abstract binding methods for controllers
}
